import json

class jsonObject(object):
	def json(self):
		return json.dumps(self.__dict__, default=lambda o: o.__dict__,
			sort_keys=True, indent=4, separators=(',', ': '))


class minervaSite(jsonObject):
	def __init__(self, baseURL='https://horizon.mcgill.ca/pban1/'):
		self.base = baseURL
		self.login = baseURL + 'twbkwbis.P_ValLogin'
		self.logout = baseURL + 'twbkwbis.P_Logout'
		self.search = baseURL + 'bwckgens.p_proc_term_date'
		self.transcript = baseURL + 'bzsktran.P_Display_Form?user_type=S&tran_type=V'
		self.quick_search = baseURL + 'bwskfreg.P_AltPin'
		self.result = baseURL + 'twbkwbis.P_Logout'


class minervaCred(jsonObject):
	def __init__(self, _mail, _password):
		self.usermail = _mail
		self.password = _password
		self.loggedin = False
		self.cookie = None

	def __repr__(self):
		return '<Credentials of: %s>' % self.usermail


class minervaCourse(jsonObject):
	def __init__(self, _subject):
		self.grade = None
		self.title = None
		self.number = None
		self.earned = None
		self.subject = _subject
		self.credits = None
		self.remarks = None
		self.average = None

	def __repr__(self):
		return "<Course: {0} ({1}cr) {2}>".format(self.subject, self.credits, self.grade)

	def __str__(self):
		_course = self
		_string = '\n    ' + repr(_course)
		return _string

	def addCourse(self, course):
		self.courses.append(course)


class minervaSemester(jsonObject):
	def __init__(self, _semester, _year):
		self.name = ' '.join((_year, _semester))
		self.year = _year
		self.courses = []
		self.semester = _semester
		self.standing = None
		self.advanced = []
		self.description = None

	def __repr__(self):
		return "<Semester: {0} ({1} courses)>".format(self.name, len(self.courses))

	def __str__(self):
		_semester = self
		_string = '\n  ' + repr(_semester)

		for _course in _semester.courses:
			_string += str(_course)

		return _string

	def addCourse(self, course):
		self.courses.append(course)


class minervaCurriculum(jsonObject):
	def __init__(self, _education='', _diploma=''):
		self.education = _education
		self.semesters = []
		self.diploma = _diploma
		
	def __repr__(self):
		return "<Curriculum: {0} ({1} semesters)>".format(self.diploma, len(self.semesters))

	def __str__(self):
		_curriculum = self
		_string = repr(_curriculum)

		for _semester in _curriculum.semesters:
			_string += str(_semester)

		return _string

	def addSemester(self, semester):
		self.semesters.append(semester)

	def lastSemester(self):
		if not self.semesters:
			sem = minervaSemester('Empty','Semester')
			self.addSemester(sem)
		return self.semesters[-1]


import minervashadow.utils.structures as structures
import identify


def _course_missing_avg(line):
	if len(line) < 8:
		return not identify._is_a_grade(line[-1])
	else:
		return False


def _course_missing_remarks(line):
	if 5 < len(line) and len(line) < 8:
		if identify._is_a_grade(line[-1]) and line[-2].isdigit():
			return True
	else:
		return False


def _course_missing_credits(line):
	if 5 < len(line) and len(line) < 8:
		for field in line[5:]:
			if field.isdigit():
				return False
		return True
	return False


def _is_valid(line):

	# Checks if the size has been fixed
	if not len(line) is 8:
		return False
	
	# Checks if the grade field is actually a grade
	if not identify._is_a_grade(line[4]):
		return False

	#Checks if the earned field has been fixed
	if not line[6].isdigit():
		return False

	# Checks the the average is a valid grade, and not blank
	if not identify._is_a_grade(line[7]):
		return False

	return True


def _fix(line):

	_fixed_line = list(line)

	if _course_missing_avg(line):
		_fixed_line.append('--')
		
	if _course_missing_credits(line):
		_fixed_line.insert(-1, u'0')
		
	if _course_missing_remarks(_fixed_line):
		_fixed_line.insert(-2, u'')
		
	return _fixed_line


def from_line(line):

	_course = structures.minervaCourse(line[0])
	
	if not _is_valid(line):
		fixed_line = _fix(line)
	else:
		fixed_line = list(line)

	if _is_valid(fixed_line):
		_course.number	= fixed_line[1]
		_course.title	= fixed_line[2]
		_course.credits	= fixed_line[3]
		_course.grade	= fixed_line[4]
		_course.remarks	= fixed_line[5]
		_course.earned	= fixed_line[6]
		_course.average = fixed_line[7]
	else:
		print line
		print 'in :', len(line), line[0], line[5:]
		print 'out:', len(fixed_line), line[0], fixed_line[5:]
		return structures.minervaCourse('Empty')
	
	return _course


from bs4 import BeautifulSoup
import login
import transcript

class MinervaReader:

	def __init__(self, site):
		self.site = site


	def login(self, html):

		"""
		Parses the result login page and extract the info message that determines a successful login.
		"""
		
		if not 'WELCOME' in html.geturl():
			return None, login.welcomeerr(html)
		else:
			return True, login.welcomemsg(html)


	def logout(self, html):
		"""
		Parses the content of the logout page and returns the message parsed.
		"""

		return 'You were successfully logged out.'


	def transcript(self, html, semester='all'):
		"""
		Parses the transcript page to extract the transcript as a json object.
		"""

		soup = BeautifulSoup(html)

		# Gets the div containing the table
		div = soup.body.find('div', {'class':'pagebodydiv'})

		# Gets the table inside that div
		tbl = div.find('table', {'class':'dataentrytable', 'width':'100%'})

		# Turns the html table into a list of lists (2D list)
		table = [col.findAll('td') for col in tbl.findAll('tr')]

		curriculum = transcript.get_curriculum(table)
		
		return curriculum


VERSION = (0, 0, 4, 'alpha', 1)

def get_version(version=None):
	"""
	Returns a PEP 386-compliant version number from VERSION.
	"""
	
	main_version, sub_version = '', ''

	if version is None:
		version = VERSION
	else:
		assert len(version) == 5
		assert version[3] in ('alpha', 'beta', 'rc', 'final')
		

	parts = 2 if version[2] == 0 else 3
	main_version = '.'.join(str(x) for x in version[:parts])

	if version[3] != 'final':
		mapping = {'alpha': 'a', 'beta': 'b', 'rc': 'c'}
		sub_version = mapping[version[3]] + str(version[4])
	else:
		pass

	return str(main_version + sub_version)

def main():
	from __main__ import main
	main()


from minervashadow.utils import structures
import identify
import course
import re

def _clean_cell(cell):
		
	cell = cell.replace(u'\xa0', u'\n') # Removes weird characters
	cell = cell.replace(u'\xb2', u'')   # Removes the pow(2) character
	#cell = cell.strip().encode('utf8')	# Removes trailing spaces
	cell = ' '.join(cell.split())		# Removes useless '\n'

	return cell


def _clean_html_table(tbl):

	# Gets rid of the html formatting to get only text
	text_matrix = [[_clean_cell(cell.text) for cell in col] for col in tbl]

	# Gets rid of empty cells
	filtered = [[cell for cell in col if cell != ''] for col in text_matrix]
	
	# Gets rid of empty rows
	filtered = [col for col in filtered if col]
	
	return filtered


def _curriculum_from_clean_table(table):

	rows = iter(table)
	headers = [col for col in next(rows)]
	standing_headers = []

	curriculum = structures.minervaCurriculum()

	for line in table:
		l = len(line)

		if l == 1:
			if identify._is_semester_header(line):
				# Gets rid of the Readmitted word before the semester.
				title = re.sub('^Readmitted', '', line[0]).split(' ')
				sem = structures.minervaSemester(title[0],title[1])
				
				curriculum.addSemester(sem)

			elif identify._is_standing(line):
				curriculum.lastSemester().standing = re.sub('^Standing: ', '', line[0])

			elif identify._is_education(line):
				education = re.sub('^PREVIOUS EDUCATION ', '', line[0])
				curriculum.education = education

			elif identify._is_description(line):
				# Puts spaces before capital letters.
				description = re.sub('(\B[A-Z])', r' \1', line[0])
				curriculum.description = description
			else:
				#print l, 'Semester Name++', line
				pass

		elif l == 4:
			#print l, 'Exemption Ignored', line
			pass

		elif l == 5:
			if identify._is_standing_header(line):
				if not standing_headers:
					standing_headers = line
			else:
				# Credits transfer and course equivalence
				#print l, 'Unknown', line
				pass

		elif l == 6 or l == 7:
			_course = course.from_line(line)
			curriculum.lastSemester().addCourse(_course)

		elif l == 8:
			pass# print l, 'TERM GPA', line
		elif l == 9:
			pass# print l, 'CUM GPA', line
		elif l == 23:
			pass# print l, 'Advanced Standing', line

	return curriculum


def get_curriculum(table):

	table = _clean_html_table(table)

	return _curriculum_from_clean_table(table)



"""
Global minervashadow exception and warning classes.
"""

class MinervaException(Exception):
	"""A generic exception for minervashadow."""
	pass


class InvalidUsername(MinervaException):
	def __init__(self, username):
		self.username = username
	def __str__(self):
		return repr(self.username)


class NoInternetConnection(MinervaException):
	"""No internet connection has been detected."""
	def __str__(self):
		return 'No internet connection has been detected.'


class AuthenticationRequired(MinervaException):
	"""You need to login to minerva before accessing this info."""
	def __str__(self):
		return 'You need to login to minerva before accessing this info.'


from utils import exceptions as ex
import utils.structures as structures
from reader import reader
import writer
import urllib2


def internet_required(fn):
	def wrapper(*args, **kwargs):
		try:
			return fn(*args, **kwargs)
		except urllib2.URLError:
			raise ex.NoInternetConnection
	return wrapper


def login_required(fn):
	def wrapper(self, *args, **kwargs):
		if not self.user.loggedin:
			raise ex.AuthenticationRequired
		return fn(self, *args, **kwargs)
	return wrapper


class MinervaSession:

	def __init__(self, user):

		self.site = structures.minervaSite()
		self.user = structures.minervaCred(user[0],user[1])
		self.writer = writer.MinervaWriter(self.site)
		self.reader = reader.MinervaReader(self.site)


	@internet_required
	def login(self) :

		# Logs in, This page can confirm a succesful login.
		loginResponse = self.writer.login(self.user)

		loggedin, msg = self.reader.login(loginResponse)

		self.user.loggedin = loggedin

		return msg[0]


	@internet_required
	def logout(self):
		
		if not self.user.loggedin:
			message = 'You are already logged out.'
		else:
			# Logs out.
			logoutPage = self.writer.logout()
			message = self.reader.logout(logoutPage)
		
		return message


	@internet_required
	@login_required
	def transcript(self):

		transcriptPage = self.writer.transcript()
		
		curriculum = self.reader.transcript(transcriptPage)

		return curriculum


	@internet_required
	def deal_with_request(self, req):

		response = ''

		if req['login']:
			response = self.login()

		elif req['transcript']:
			response = self.transcript()

		else:
			response = 'Not implemented yet.'

		return response


#!/usr/bin/env python
"""
Usage:
	minervashadow login
	minervashadow transcript
	minervashadow -h | --help
	minervashadow -v | --version

Examples:
	minervashadow login
	minervashadow transcript

Options:
	-h, --help     Show this screen.
	-v, --version  Print the current version.
"""

from minervashadow import get_version
from docopt import docopt
import ui.cli as cli
import minerva


def interruptable(fn):
	"""Exits the program if Ctrl+C is pressed"""
	def wrapper(*args, **kwargs):
		try:
			return fn(*args, **kwargs)
		except KeyboardInterrupt:
			print '\n\nExiting...\nHope to see you soon!'
			exit()
	return wrapper


@interruptable
def main():

	args = docopt(__doc__, version=get_version())

	credentials = cli.get_user_credentials()

	session = minerva.MinervaSession(credentials)

	print session.login()
	
	if not args['login']:
		print session.deal_with_request(args)

	session.logout()


if __name__ == '__main__':
	main()


import re

def _is_semester_header(line):
	sem_header_regex = re.compile('^(Readmitted|)(Fall|Winter|Summer) ([0-9]{4})$')
	return not not sem_header_regex.match(line[0])

def _is_standing(line):
	return line[0].startswith('Standing: ')

def _is_education(line):
	return line[0].startswith('PREVIOUS EDUCATION ')

def _is_description(line):
	return line[0].startswith('Bachelor of ')

def _is_standing_header(line):
	return line[0] == 'Advanced Standing& Transfer Credits:'

def _is_a_grade(text):
	rule = '^([A-C][+-]?|D|F'
	rule += '|HH|IP|JK|KE|K\*|KF|KK|LL|LE|L\*'
	rule += '|NA|&&|NE|NR|P|Q|R|S|U|W|WF|WL|W--|--)$'
	grade_regex = re.compile(rule)
	return not not grade_regex.match(text)

if __name__ == '__main__':
	print 'B+', _is_a_grade(u'B+')
	print 'K*', _is_a_grade(u'K*')
	print '3',  _is_a_grade(u'3')


try:
	from setuptools import setup 
except:
	from disutils.core import setup

import minervashadow


dependencies=['docopt','mechanize','beautifulsoup4']

setup(

	name='minervashadow',
	version=minervashadow.get_version(),
	description='A python interface to the aging minerva website.',
	long_description=open('README.md').read(),
	url='https://github.com/cadesalaberry/minervashadow',
	author='cadesalaberry',
	author_email='cadesalaberry@yahoo.com',
	install_requires=dependencies,
	packages=[
		'minervashadow','minervashadow.reader',
		'minervashadow.utils','minervashadow.ui'
		],
	entry_points={
		'console_scripts': [
			'minervashadow=minervashadow:main'
		],
	},
	classifiers=[
		'Development Status :: 3 - Alpha',
		'Intended Audience :: Developers',
		'Natural Language :: French',
		'License :: OSI Approved :: MIT License',
		'Programming Language :: Python',
		'Programming Language :: Python :: 2.6'
	]
)



import utils.calendar as calendar
import mechanize

class MinervaWriter:

	def __init__(self, site):
		self.site = site
		self.bwsr = mechanize.Browser()


	def login(self, user):

		# Deals with the login page
		loginPage = self.bwsr.open(self.site.login)
		self.bwsr.select_form(name='loginform1')
		self.bwsr['sid'] = user.usermail
		self.bwsr['PIN'] = user.password

		return self.bwsr.submit()


	def setsemester(self, semester):

		# Selects the right semester on the semester page.
		self.bwsr.open(self.site.quick_search)
		self.bwsr.select_form(nr=1)
		self.bwsr['term_in'][0] = semester

		return self.bwsr.submit()	


	def drop(self, crn, semester=calendar.curr_semester()):

		print 'The class was not actually dropped, missing implementation.'
		return self.setsemester(semester)


	def transcript(self):

		return self.bwsr.open(self.site.transcript)


	def logout(self):

		return self.bwsr.open(self.site.logout)



from HTMLParser import HTMLParser
import urlparse

def welcomemsg(html):
	query = urlparse.parse_qs(html.geturl())
	messg = HTMLParser().unescape(query['msg'][0])
	messg = messg.replace('<b>', '').replace('WELCOME ', '')
	return messg.split('</b>')

def welcomeerr(html):
	# TODO:Instead we should try reading the error message displayed on the html
	return ['Authorization Failure - you have entered an invalid McGill Username / Password.']


from minervashadow.utils.exceptions import InvalidUsername
from getpass import getpass
import sys
import os


def get_user_credentials():

	"""Gets the user credentials from the commandline.

	Asks the user for his McGill email address and password.


	Args:
		None

	Returns:
		An array containing the user credentials.

		Notice that no checks for validity is applied to the input.
		For testing purposes, if not executed in a tty, the username
		and password are goign to be read from stdin.

		Example:

			['username','password']

	Raises:
		IOError: An error occurred accessing the username file.
	"""

	if sys.stdin.isatty():
		# Gets the credentials from the userFile if it exists
		_mail = getusername()
		_pass = getpass(stream=sys.stderr)
		_cred = [_mail, _pass]

	else:
		# Gets the credentials from stdin
		_cred = sys.stdin.readlines()
		_cred = map(str.strip, _cred)

		print 'autologin\t:', _cred[0] 
		if len(_cred) != 2:
			print 'Error: Malformed input. Missing arguments.'
			print 'Here is what your input should look like:'
			print '\tjonh.doe@mail.mcgill.ca'
			print '\tpassword'
			exit()
			
	return _cred


def getusername(reset=False):

	"""	Gets the username either from the commandline,
		either from the user file (in ~/.minerva/).

	Asks the user for his McGill email address if no previous session
	has been stored, and save the session in the user file.


	Args:
		None

	Returns:
		A string containing the user email.

		Example:

			'john.doe@mail.mcgill.ca'

	Raises:
		IOError: An error occurred accessing the username file.
	"""

	def _is_valid(username):
		return ('@' and '.' in username)

	_mailSaved = savedusername()
	_mail = ""
	
	if _is_valid(_mailSaved) and not reset:
		print 'Read\t:', _mailSaved
		_mail = _mailSaved
	else:
		_mail = raw_input('Email\t: ')

		# Auto-completes the username if domain missing 
		if '.' in _mail and not '@' in _mail:
			_mail = _mail + '@mail.mcgill.ca'

		if not _is_valid(_mail):
			raise InvalidUsername(_mail)

	# Stores the email to the user file
	if _mail != _mailSaved:
		savedusername(_mail)

	return _mail


def savedusername(username=""):

	"""
	Reads the email address stored in the userfile.
	if the file does not exist, it creates it.

	It also stores the username passed as argument if it is not empty.


	Args:
		A username as a string.

		Example:

			'john.doe@mail.mcgill.ca'

	Returns:
		Either a string containing the username that has been read,
		either a string containing the username that has just been stored.

		Example:

			'john.doe@mail.mcgill.ca'

	Raises:
		IOError: An error occurred accessing the username file.
	"""

	_userPath = os.path.expanduser("~/.minerva/")
	_userFile = os.path.join(_userPath, 'user')

	if not os.path.exists(_userPath):
		os.makedirs(_userPath)

	if not os.path.exists(_userFile):
		open(_userFile, 'w').close()

	if username == "":
		userFile = open(_userFile, 'r+')
		username = userFile.readline().strip()
		userFile.close()
	else:
		userFile = open(_userFile, 'w+')
		userFile.write(username)
		userFile.close()

	return username